﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace GuiClient
{
    /// <summary>
    /// フォーム上のリッチテキストボックス更新のためのdelegate
    /// </summary>
    /// <param name="s"></param>
    delegate void SamDelegate(string s);

    public partial class Form1 : Form
    {
        TcpClient client;
        NetworkStream netStream;
        StreamReader sr;
        StreamWriter sw;
        const string CRLF = "\r\n";

        public Form1()
        {
            InitializeComponent();

            buttonSend.Enabled = false;
            buttonStop.Enabled = false;
        }


        public void SetText(string s)
        {
            richTextBoxShow.AppendText(s + CRLF);
        }

        /// <summary>
        /// スタートボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonStart_Click(object sender, EventArgs e)
        {
            try
            {
                client = new TcpClient(textBoxIP.Text, int.Parse(textBoxPort.Text));

                netStream = client.GetStream();

                sr = new StreamReader(netStream, Encoding.UTF8);
                sw = new StreamWriter(netStream, Encoding.UTF8);
                sw.AutoFlush = true;

                buttonStart.Enabled = false;
                buttonStop.Enabled = true;
                buttonSend.Enabled = true;

                SetText("接続しました");
            }
            catch
            {
                SetText("接続できません");
            }
        }

        /// <summary>
        /// ストップボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonStop_Click(object sender, EventArgs e)
        {

            try
            {
                sr.Close();
                sw.Close();
                netStream.Close();
                client.Close();
                SetText("コネクションを切断しました");
            }
            catch
            {
                SetText("コネクションの切断操作は失敗しました");
            }

            buttonStart.Enabled = true;
            buttonStop.Enabled = false;
            buttonSend.Enabled = false;
        }

        /// <summary>
        /// 送信ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSend_Click(object sender, EventArgs e)
        {
            SendData(textBoxMsg.Text);
        }

        /// <summary>
        /// 送信スレッドを起動するメソッド
        /// </summary>
        /// <param name="s">送信する文字列</param>
        private void SendData(string s)
        {
            WorkProce wp = new WorkProce(this);
            wp.SR = sr;
            wp.SW = sw;
            wp.Data = s;

            Thread t = new Thread(new ThreadStart(wp.SendToServer));
            t.IsBackground = true;
            t.Start();
        }
    }

    /// <summary>
    /// 送受信するクラス
    /// </summary>
    public class WorkProce
    {
        private Form1 _form;

        private StreamReader _sr;
        private StreamWriter _sw;
        private string _data;

        /// <summary>
        /// 受信用StreamReader
        /// </summary>
        public StreamReader SR
        {
            set { _sr = value; }
        }

        /// <summary>
        /// 送信用StreamWriter
        /// </summary>
        public StreamWriter SW
        {
            set { _sw = value; }
        }

        /// <summary>
        /// 送信データの文字列
        /// </summary>
        public string Data
        {
            set { _data = value; }
        }

        public WorkProce(Form1 form)
        {
            _form = form;
        }

        public void SendToServer()
        {
            try
            {
                _sw.WriteLine(_data);
                string s = _sr.ReadLine();
                _form.Invoke(new SamDelegate(_form.SetText), s);
            }
            catch
            {
                _form.Invoke(new SamDelegate(_form.SetText), "送信できませんでした");
            }
        }
    }
}
