﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Collections;

namespace GuiServer
{
    /// <summary>
    /// UIスレッド中のメソッドを使うためのdelegate
    /// </summary>
    /// <param name="obj"></param>
    delegate void SamDelegate(object obj);

    public partial class Form1 : Form
    {
        const string CRLF = "\r\n";
        public volatile bool flag = true;
        ArrayList Session = new ArrayList();

        private TcpListener listener;
        Thread t;

        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// メッセージを表示します
        /// </summary>
        /// <param name="obj"></param>
        public void SetText(object obj)
        {
            rTboxShow.AppendText((string)obj + CRLF);
        }

        /// <summary>
        /// 接続をリストに追加します
        /// </summary>
        /// <param name="obj"></param>
        public void AddSession(object obj)
        {
            Session.Add(obj);
        }

        /// <summary>
        /// 接続をリストから削除します
        /// </summary>
        /// <param name="obj"></param>
        public void RemoveSession(object obj)
        {
            for (int i = 0; i < Session.Count; i++)
            {
                try
                {
                    if (((WorkerProc)Session[i]).ID == ((WorkerProc)obj).ID)
                    {
                        Session.RemoveAt(i);
                        break;
                    }
                }
                catch
                {
                }
            }
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            btnStop.Enabled = true;
            btnStart.Enabled = false;

            flag = true;

            /// サーバをメインスレッドと別のスレッドで起動します
            t = new Thread(new ThreadStart(StartServer));
            t.IsBackground = true;
            t.Start();
        }

        private void StartServer()
        {
            IPAddress addr = null;

            /// ローカルIPアドレスを取得します
            try
            {
                for (int i = 0; i < Dns.GetHostEntry(Dns.GetHostName()).AddressList.Length; i++)
                {
                    if (Dns.GetHostEntry(Dns.GetHostName()).AddressList[i].AddressFamily == AddressFamily.InterNetwork)
                    {
                        addr = Dns.GetHostEntry(Dns.GetHostName()).AddressList[i];
                        break;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return;
            }

            IPEndPoint localEP = new IPEndPoint(addr, 8888);

            listener = new TcpListener(localEP);

            /// リスナを開始します
            try
            {
                listener.Start(10);
            }
            catch (Exception ex)
            {
                rTboxShow.AppendText(ex.Message + CRLF);
                return;
            }

            while (flag)
            {
                if (listener.Pending())
                {
                    /// キューから送受信用のTcpClientを生成します
                    TcpClient client = listener.AcceptTcpClient();

                    if (client.Connected)
                    {
                        /// 送受信するスレッドを起動します
                        WorkerProc proc = new WorkerProc(this);
                        proc.Client = client;
                        proc.ID = Guid.NewGuid();

                        this.Invoke(new SamDelegate(this.AddSession), proc);

                        Thread t = new Thread(new ThreadStart(proc.Run));
                        t.IsBackground = true;
                        t.Start();
                    }
                }

                Thread.Sleep(50);
            }

            /// リスナをストップします
            listener.Stop();
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            btnStart.Enabled = true;
            btnStop.Enabled = false;

            flag = false;


            for (int i = Session.Count - 1; i >= 0; i--)
            {
                try
                {
                    ((WorkerProc)Session[i]).Client.Close();
                }
                catch
                {
                }
            }
        }
    }

    /// <summary>
    /// データ受信クラス
    /// </summary>
    public class WorkerProc
    {
        Form1 _form;
        TcpClient _client;
        Guid id;

        public WorkerProc(Form1 form)
        {
            _form = form;
        }

        public TcpClient Client
        {
            set { _client = value; }
            get { return _client; }
        }

        public Guid ID
        {
            get { return this.id; }
            set { this.id = value; }
        }

        public void Run()
        {
            NetworkStream netStream = this._client.GetStream();

            IPEndPoint ipe = (IPEndPoint)this._client.Client.RemoteEndPoint;

            /// クライアントからの接続要求が確立できたため、UIスレッドの知らせます
            _form.Invoke(new SamDelegate(_form.SetText), "Connected from " + ipe.Address.ToString());

            StreamReader sr = new StreamReader(netStream, Encoding.UTF8);
            StreamWriter sw = new StreamWriter(netStream, Encoding.UTF8);
            sw.AutoFlush = true;

            string s = String.Empty;

            /// メッセージ受信ループ
            /// 終了条件は:
            /// 1: フォーム上の「停止ボタン」を押される、送受信用TcpClientが閉じられた時
            /// 2: クライアントから quitコマンドを送られてきた時
            /// 3: ネットワークダウン(クライアントとの接続) 
            /// 4: ネットワークストリームからデータの読み取りはエラーになる時
            do
            {
                try
                {
                    s = sr.ReadLine();

                    if (s == null)
                        break;

                    sw.WriteLine("Server>>>" + s);
                    _form.Invoke(new SamDelegate(_form.SetText), s);

                }
                catch
                {
                    break;
                }
            }
            while (!s.Equals("quit"));

            sw.Close();
            sr.Close();
            _client.Close();

            _form.Invoke(new SamDelegate(_form.RemoveSession), this);

            /// クライアントとの接続が切れたことをUIに知らせます
            _form.Invoke(new SamDelegate(_form.SetText), "Disconnected " + ipe.Address.ToString());
        }
    }
}
