﻿//////////////////////////////////////////
///
/// NetControl Ver: 1.5
/// Client Project
/// Copyright Sam Zhao
/// 2007 Tokyo Japan
///
////////////////////////////////////////// 
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Diagnostics;
using System.Collections;


namespace NetClient
{
    delegate void Samdelegate(object obj);

    public partial class Form1 : Form
    {
        CLIENT client;
        public string FilePath = String.Empty;

        public Form1()
        {
            InitializeComponent();
        }

        public void SetText(object obj)
        {
            rTboxShow.AppendText((string)obj + "\r\n");
        }

        public void SetClientIpText(object obj)
        {
            this.Text = "Client -> " + (string)obj;
        }

        public void FileReceiveFinish(object obj)
        {
            MessageBox.Show((string)obj);
        }

        public void SetScanStat(object obj)
        {
            textBoxStatus.Text = (int.Parse(textBoxStatus.Text) + 1).ToString();
        }
        /// <summary>
        /// フォールロードされた時点で、ローカルIPアドレスを表示します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {
            foreach (IPAddress addr in Dns.GetHostAddresses(Dns.GetHostName()))
            {
                if (addr.AddressFamily == AddressFamily.InterNetwork)
                {
                    this.Text = "Client-> " + addr.ToString();
                    break;
                }
            }

            /// 接続確立する前にすべてのボタンを無効にします
            EnableButtons(false);
        }

        /// <summary>
        /// ボタン有効無効制御
        /// </summary>
        /// <param name="enable"></param>
        private void EnableButtons(bool enable)
        {
            btnStart.Enabled = enable ? false : true;
            btnStop.Enabled = enable ? true : false;

            btnMsg.Enabled = enable ? true : false;
            btnCD.Enabled = enable ? true : false;
            btnShowForm.Enabled = enable ? true : false;
            btnDriv.Enabled = enable ? true : false;
            btnVer.Enabled = enable ? true : false;
            btnScreen.Enabled = enable ? true : false;
            btnRun.Enabled = enable ? true : false;
            btnGotoURL.Enabled = enable ? true : false;
            btnFile.Enabled = enable ? true : false;
            btnShowForm.Enabled = enable ? true : false;
            btnFormHide.Enabled = enable ? true : false;
        }

        /// <summary>
        /// サーバに接続します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStart_Click(object sender, EventArgs e)
        {
            try
            {
                string serverip = textBoxIP.Text;
                int port = int.Parse(textBoxPort.Text);
                client = new CLIENT(this);

                client.ServerIP = serverip;
                client.Port = port;
                if (client.ConnectToServer())
                {
                    Thread t = new Thread(new ThreadStart(client.Receive));
                    t.IsBackground = true;
                    t.Start();
                    EnableButtons(true);
                }
            }
            catch
            {
            }
        }

        /// <summary>
        /// 接続を切断します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStop_Click(object sender, EventArgs e)
        {
            try
            {
                client.CloseClient();
            }
            catch
            {
            }

            EnableButtons(false);
        }

        /// <summary>
        /// サーバにメッセージを送信します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnMsg_Click(object sender, EventArgs e)
        {
            try
            {
                client.SendToServer("msg " + textBoxMsg.Text);
            }
            catch { }
        }

        /// <summary>
        /// サーバのCDドライブを制御します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCD_Click(object sender, EventArgs e)
        {
            try
            {
                client.SendToServer("cdcontrol ");
            }
            catch
            { }
        }


        /// <summary>
        /// サーバの物理ドライブ情報を取得します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnDriv_Click(object sender, EventArgs e)
        {
            try
            {
                client.SendToServer("driverinfo ");
            }
            catch
            { }
        }

        /// <summary>
        /// サーバのWindowsバージョンを取得します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnVer_Click(object sender, EventArgs e)
        {
            try
            {
                client.SendToServer("winversion ");
            }
            catch
            { }
        }

        /// <summary>
        /// サーバのスクリーン画面を取得します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnScreen_Click(object sender, EventArgs e)
        {
            try
            {
                SERVER server = new SERVER(true);
                server.FORM = this;
                server.Port = 1889;
                server.Path = Application.StartupPath + @"\tmp.jpg";

                Thread tFile = new Thread(new ThreadStart(server.ReceiveFile));
                tFile.IsBackground = true;
                tFile.Start();

                client.SendToServer("getscreen ");
            }
            catch
            { }
        }

        /// <summary>
        /// 指定したアプリケーションを起動します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnRun_Click(object sender, EventArgs e)
        {
            try
            {
                client.SendToServer("run " + textBoxAppName.Text);
            }
            catch
            { }
        }

        /// <summary>
        /// 指定したURLをオープンします
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnGotoURL_Click(object sender, EventArgs e)
        {
            try
            {
                client.SendToServer("url " + textBoxURL.Text);
            }
            catch
            { }
        }

        /// <summary>
        /// サーバ上のファイルを操作します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFile_Click(object sender, EventArgs e)
        {
            try
            {
                string path = textBoxPath.Text;
                if (radioButtonList.Checked)
                {
                    client.SendToServer("list " + path);
                }
                if (radioButtonDel.Checked)
                {
                    client.SendToServer("del " + path);
                }
                if (radioButtonCopy.Checked)
                {
                    InputForm input = new InputForm(this);
                    if (input.ShowDialog() == DialogResult.OK)
                    {
                        SERVER server = new SERVER(false);
                        server.FORM = this;
                        server.Port = 1889;
                        server.Path = FilePath;

                        Thread tFile = new Thread(new ThreadStart(server.ReceiveFile));
                        tFile.IsBackground = true;
                        tFile.Start();

                        client.SendToServer("copy " + path);
                    }
                }
            }
            catch
            { }
        }

        /// <summary>
        /// スキャナーを起動します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnScan_Click(object sender, EventArgs e)
        {
            textBoxStatus.Text = "0";

            try
            {
                ArrayList list = new ArrayList();
                list.Add(this);
                list.Add(textBoxScanIP.Text);
                list.Add(textBoxScanPort.Text);
                if (!bwScan.IsBusy)
                {
                    bwScan.RunWorkerAsync(list);
                }
            }
            catch
            { }
        }

        /// <summary>
        /// スキャンプロセスを起動します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bwScan_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                ArrayList list = (ArrayList)e.Argument;
                Form1 form = (Form1)list[0];
                string startip = (string)list[1];
                int port  = int.Parse((string)list[2]);

                for (int i = 1; i < 255; i++)
                {
                    SCAN scan = new SCAN(form);
                    scan.IP = startip + "." + i.ToString();
                    scan.Port = port;

                    ThreadPool.QueueUserWorkItem(new WaitCallback(scan.StartScan), null);
                }
            }
            catch
            {
            }
        }

        /// <summary>
        /// 切断ボタンを押さずフォームを閉じる時にまずコネクションを切ります
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                client.CloseClient();
            }
            catch { }
        }

        /// <summary>
        /// サーバのフォームを表示します。
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnShowForm_Click(object sender, EventArgs e)
        {
            try
            {
                client.SendToServer("ShowForm ");
            }
            catch
            {
            }
        }

        /// <summary>
        /// サーバのフォームを隠します
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnFormHide_Click(object sender, EventArgs e)
        {
            try
            {
                client.SendToServer("HideForm ");
            }
            catch
            {
            }
        }
    }

    /// <summary>
    /// スキャナークラス
    /// </summary>
    public class SCAN
    {
        private string _ip;
        /// <summary>
        /// スキャン先のIPアドレス
        /// </summary>
        public string IP
        {
            set { _ip = value; }
        }
        private int _port;
        /// <summary>
        /// スキャン先のポート番号
        /// </summary>
        public int Port
        {
            set { _port = value; }
        }

        private Form1 _form;
        public SCAN(Form1 form)
        {
            _form = form;
        }

        /// <summary>
        /// スキャン
        /// </summary>
        /// <param name="obj"></param>
        public void StartScan(object obj)
        {
            try
            {
                TcpClient client = new TcpClient(_ip, _port);
                client.Close();
                _form.Invoke(new Samdelegate(_form.SetText), _ip + ":" + _port +" open");
            }
            catch
            {
            }

            _form.Invoke(new Samdelegate(_form.SetScanStat), "");
        }
    }

    /// <summary>
    /// 受信クラス
    /// </summary>
    public class SERVER
    {
        private Form1 _form;
        public Form1 FORM
        {
            set { _form = value; }
        }

        private bool open = false;

        private int _port;
        /// <summary>
        /// データ受信待機ポート番号
        /// </summary>
        public int Port
        {
            set { _port = value; }
        }

        private string _path;
        /// <summary>
        /// ファイルのフルパス
        /// </summary>
        public string Path
        {
            set { _path = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mode">ファイルを開くかしないか</param>
        public SERVER(bool mode)
        {
            open = mode;
        }

        /// <summary>
        /// ファイル受信プロセス、画像ファイルの場合、受信した後、
        /// 画像を表示します。
        /// </summary>
        public void ReceiveFile()
        {
            try
            {
                TcpListener listener = new TcpListener(GetLocalIPAddress(), _port);
                listener.Start(1);
                TcpClient client = listener.AcceptTcpClient();

                NetworkStream netStream = client.GetStream();

                byte[] bb = new byte[1024];
                int readSize = 0;
                FileStream fs = new FileStream(_path, FileMode.Create, FileAccess.Write);


                while ((readSize = netStream.Read(bb, 0, bb.Length)) > 0)
                {
                    fs.Write(bb, 0, readSize);
                    fs.Flush();
                }

                fs.Close();

                netStream.Close();
                client.Close();
                listener.Stop();

                /// 画像ファイルの場合は表示します
                if (open)
                {
                    try
                    {
                        Process.Start(_path);
                    }
                    catch { }
                }
                else
                {
                    _form.Invoke(new Samdelegate(_form.FileReceiveFinish), "ファイル転送完了！");
                }
            }
            catch
            {
                _form.Invoke(new Samdelegate(_form.FileReceiveFinish), "転送失敗");
            }
        }

        /// <summary>
        /// ローカルIPアドレスを取得します
        /// </summary>
        /// <returns></returns>
        private IPAddress GetLocalIPAddress()
        {
            foreach (IPAddress addr in Dns.GetHostAddresses(Dns.GetHostName()))
            {
                if (addr.AddressFamily == AddressFamily.InterNetwork)
                    return addr;
            }
            return IPAddress.Parse("127.0.0.1");
        }

    }

    /// <summary>
    /// 送信クラス
    /// </summary>
    public class CLIENT
    {
        private TcpClient _client;
        private NetworkStream _netStream;
        private StreamWriter sw;
        private StreamReader sr;

        private string _ip;
        /// <summary>
        /// リモートサーバのIPアドレス
        /// </summary>
        public string ServerIP
        {
            set { _ip = value; }
        }

        private int _port;
        /// <summary>
        /// TCPポート
        /// </summary>
        public int Port
        {
            set { _port = value; }
        }

        private Form1 _form;
        public CLIENT(Form1 form)
        {
            _form = form;
        }

        /// <summary>
        /// サーバからテキストメッセージを受信します
        /// </summary>
        public void Receive()
        {
            while (true)
            {
                try
                {
                    string s = sr.ReadLine();
                    
                    if (s == null)
                    {
                        break;
                    }

                    _form.Invoke(new Samdelegate(_form.SetText), s);
                }
                catch
                {
                    break;
                }
            }
            this.CloseClient();
        }

        /// <summary>
        /// サーバに接続します
        /// </summary>
        public bool ConnectToServer()
        {
            try
            {
                _client = new TcpClient(_ip, _port);
                _netStream = _client.GetStream();
                sw = new StreamWriter(_netStream, Encoding.UTF8);
                sr = new StreamReader(_netStream, Encoding.UTF8);

                _form.Invoke(new Samdelegate(_form.SetClientIpText), _ip);

                return true;
            }
            catch { return false; }
        }

        /// <summary>
        /// サーバにテキストメッセージを送信します
        /// </summary>
        /// <param name="msg"></param>
        public bool SendToServer(string msg)
        {
            try
            {
                sw.WriteLine(msg);
                sw.Flush();
                return true;
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// コネクションを切ります
        /// </summary>
        public void CloseClient()
        {
            try
            {
                sr.Close();
                sw.Close();
                _netStream.Close();
                _client.Close();
            }
            catch
            {
            }
        }
    }
}
