﻿//////////////////////////////////////////
///
/// NetControl Ver: 1.5
/// Server Project
/// Copyright Sam Zhao
/// 2007 Tokyo Japan
/// 
//////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Collections;
using System.Threading;
using System.Runtime.InteropServices;
using System.Drawing.Imaging;
using System.Diagnostics;
using System.Management;

namespace NetServer
{
    delegate void SamDelegate(object obj);

    public partial class Form1 : Form
    {
        const string CRLF = "\r\n";
        /// <summary>
        /// サーバ起動、停止フラグ
        /// </summary>
        public volatile bool flag = true;
        /// <summary>
        /// すべてのコネクションの管理
        /// </summary>
        ArrayList Session = new ArrayList();

        /// <summary>
        /// メインリスナ
        /// </summary>
        TcpListener listener;

        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// サーバ起動
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStart_Click(object sender, EventArgs e)
        {
            btnStart.Enabled = false;
            btnStop.Enabled = true;

            flag = true;

            Thread tMain = new Thread(new ThreadStart(StartServer));
            tMain.IsBackground = true;
            tMain.Start();
        }

        /// <summary>
        /// サーバ停止
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStop_Click(object sender, EventArgs e)
        {
            btnStart.Enabled = true;
            btnStop.Enabled = false;

            flag = false;

            for (int i = Session.Count - 1; i >= 0; i--)
            {
                try
                {
                    ((WorkerProc)Session[i]).Client.Close();
                }
                catch
                { }
            }
        }

        public void SetText(object obj)
        {
            rTboxShow.AppendText((string)obj + CRLF);
        }

        /// <summary>
        /// 確立済みのコネクションをセッションリストに追加します
        /// </summary>
        /// <param name="obj"></param>
        public void AddSession(object obj)
        {
            Session.Add(obj);
        }

        /// <summary>
        /// 破棄済みのコネクションをセッションリストから削除します
        /// </summary>
        /// <param name="obj"></param>
        public void RemoveSession(object obj)
        {
            for (int i = 0; i < Session.Count; i++)
            {
                try
                {
                    if (((WorkerProc)Session[i]).ID == ((WorkerProc)obj).ID)
                    {
                        Session.RemoveAt(i);
                        break;
                    }
                }
                catch
                { }
            }
        }

        /// <summary>
        /// フォームを表示するかしないか
        /// </summary>
        /// <param name="obj"></param>
        public void ShowHideForm(object obj)
        {
            if ((bool)obj)
            {
                this.Visible = true;
            }
            else
            {
                this.Visible = false;
            }
        }

        /// <summary>
        /// ローカルIPv4 IPアドレスを取得します
        /// </summary>
        /// <returns>ローカルIPv4アドレス</returns>
        private IPAddress GetLocalIPAddress()
        {
            foreach (IPAddress addr in Dns.GetHostAddresses(Dns.GetHostName()))
            {
                if (addr.AddressFamily == AddressFamily.InterNetwork)
                    return addr;
            }
            return IPAddress.Parse("127.0.0.1");
        }

        /// <summary>
        /// メインリスナを起動します
        /// </summary>
        private void StartServer()
        {
            IPEndPoint localEP = new IPEndPoint(GetLocalIPAddress(), 1888);
            listener = new TcpListener(localEP);
            try
            {
                listener.Start(5);
                this.Invoke(new SamDelegate(this.SetText), "Server listening on " + GetLocalIPAddress().ToString());
            }
            catch (Exception ex)
            {
                this.Invoke(new SamDelegate(SetText), ex.Message);
                return;
            }

            while (flag)
            {
                if (listener.Pending())
                {
                    TcpClient client = listener.AcceptTcpClient();

                    if (client.Connected)
                    {
                        WorkerProc proc = new WorkerProc(this);
                        proc.Client = client;
                        proc.ID = Guid.NewGuid();

                        this.Invoke(new SamDelegate(AddSession), proc);

                        Thread t = new Thread(new ThreadStart(proc.Run));
                        t.IsBackground = true;
                        t.Start();
                    }
                }

                Thread.Sleep(50);
            }

            listener.Stop();
            this.Invoke(new SamDelegate(this.SetText), "Server Stopped");
        }
    }

    /// <summary>
    /// 通信用クラス
    /// </summary>
    public class WorkerProc
    {
        private volatile bool DoorOpened = false;

        Form1 _form;
        TcpClient _client;
        Guid _id;

        public WorkerProc(Form1 form)
        {
            _form = form;
        }

        /// <summary>
        /// 通信用TcpClient
        /// </summary>
        public TcpClient Client
        {
            set { _client = value; }
            get { return _client; }
        }

        /// <summary>
        /// コネクションの識別 GUID
        /// </summary>
        public Guid ID
        {
            set { _id = value; }
            get { return _id; }
        }

        public void Run()
        {
            NetworkStream netStream = this._client.GetStream();
            IPEndPoint ipe = (IPEndPoint)this._client.Client.RemoteEndPoint;

            _form.Invoke(new SamDelegate(_form.SetText), "Connected from " + ipe.Address.ToString());

            StreamReader sr = new StreamReader(netStream, Encoding.UTF8);
            StreamWriter sw = new StreamWriter(netStream, Encoding.UTF8);
            sw.AutoFlush = true;

            string s = String.Empty;
            string rs = string.Empty;
            do
            {
                try
                {
                    s = sr.ReadLine();

                    if (s == null)
                        break;
                    string[] ss = s.Split(' ');
                    switch (ss[0])
                    {   /// ディレクトリとファイルをリストします
                        case "list":
                            CmdList(ss[1], ref rs);
                            break;
                        /// 指定したファイルを削除します
                        case "del":
                            CmdDel(ss[1], ref rs);
                            break;
                        /// 指定したファイルをクライアントに転送します
                        case "copy":
                            CmdCopy(ss[1], ref rs);
                            break;
                        /// サーバのドライブ情報を取得します
                        case "driverinfo":
                            CmdDriverInfo(ss[1], ref rs);
                            break;
                        /// サーバにメッセージを送信します
                        case "msg":
                            CmdMsg(ss[1], ref rs);
                            break;
                        /// 指定したアプリケーションを起動します
                        case "run":
                            CmdRunApp(ss[1], ref rs);
                            break;
                        /// 指定したURLを開けます
                        case "url":
                            CmdGoToURL(ss[1], ref rs);
                            break;
                        /// サーバのバージョンを取得します
                        case "winversion":
                            CmdWinVer(ss[1], ref rs);
                            break;
                        /// CDドライブオープン・クローズ
                        case "cdcontrol":
                            CmdCdControl(ss[1], ref rs);
                            break;
                        /// フォームを表示します
                        case "ShowForm":
                            ShowForm(ss[1], ref rs);
                            break;
                        /// フォームを隠します
                        case "HideForm":
                            HideForm(ss[1], ref rs);
                            break;
                        /// サーバのスクリーン画面を取得します
                        case "getscreen":
                            CmdGetScreen(ss[1], ref rs);
                            break;
                        default:
                            break;
                    }

                    sw.WriteLine(rs);
                    _form.Invoke(new SamDelegate(_form.SetText), s);
                }
                catch
                {
                    break;
                }
            }
            while (!s.Equals("quit"));

            sw.Close();
            sr.Close();
            _client.Close();

            _form.Invoke(new SamDelegate(_form.RemoveSession), this);
            _form.Invoke(new SamDelegate(_form.SetText), "Disconnected " + ipe.Address.ToString());
        }

        /// <summary>
        /// ディレクトリとファイルのリスト
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdList(string msg, ref string result)
        {
            try
            {
                string[] str = new string[1024];
                string dir = null;
                for (int i = 0; i < Directory.GetDirectories(msg).Length; i++)
                {
                    str[i] = Directory.GetDirectories(msg)[i];
                    dir = dir + "Dir: " + "[" + str[i] + "\r\n";
                }
                result = dir;
            }
            catch (Exception ex)
            {
                result = ex.Message;
            }
        }

        /// <summary>
        /// フォームを表示します
        /// このメソッドは渡されたパラメータを使いません
        /// </summary>
        /// <param name="msg"></param>
        /// <param name="result"></param>
        private void ShowForm(string msg, ref string result)
        {
            _form.Invoke(new SamDelegate(_form.ShowHideForm), true);
            result = "フォームを表示しました。";
        }

        /// <summary>
        /// フォームを隠します
        /// このメソッドは渡されたパラメータを使いません
        /// </summary>
        /// <param name="msg"></param>
        /// <param name="result"></param>
        private void HideForm(string msg, ref string result)
        {
            _form.Invoke(new SamDelegate(_form.ShowHideForm), false);
            result = "フォームを隠しました。";
        }

        /// <summary>
        /// 指定したファイルを削除します
        /// (削除権限がある場合)
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdDel(string msg, ref string result)
        {
            try
            {
                File.Delete(msg);
                result = "削除しました。";
            }
            catch (Exception ex)
            {
                result = ex.Message;
            }
        }

        /// <summary>
        /// 指定したファイルをクライアントに転送します
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdCopy(string msg, ref string result)
        {
            try
            {
                string ip = ((IPEndPoint)_client.Client.RemoteEndPoint).Address.ToString();
                TcpClient c = new TcpClient(ip, 1889);
                NetworkStream netStream = c.GetStream();

                FileStream fs = new FileStream(msg, FileMode.Open, FileAccess.Read);
                FileInfo info = new FileInfo(msg);
                long len = info.Length;
                byte[] bb = new byte[len];
                fs.Read(bb, 0, (int)len);
                netStream.Write(bb, 0, (int)len);
                netStream.Flush();

                fs.Close();
                netStream.Close();
                c.Close();

                result = "ファイル送信終了。";
            }
            catch(Exception ex)
            {
                result = ex.Message;
            }
        }

        /// <summary>
        /// ドライブ情報を取得します
        /// このメソッドは渡されたパラメータを使いません
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdDriverInfo(string msg, ref string result)
        {
            ManagementObject mo = new ManagementObject();
            string[] driveType = Directory.GetLogicalDrives();
            string driveList = "";
            foreach (string drive in driveType)
            {
                mo.Path = new ManagementPath("Win32_LogicalDisk='" + drive.TrimEnd('\\') + "'");
                switch (Convert.ToInt32(mo.Properties["DriveType"].Value))
                {
                    case 0:
                        driveList = drive.Substring(0, 1) + "  Unknown driver type \r\n";
                        break;
                    case 1:
                        driveList = driveList + drive.Substring(0, 1) + "  no root dir\r\n";
                        break;
                    case 2:
                        driveList = driveList + drive.Substring(0, 1) + "  Removable \r\n";
                        break;
                    case 3:
                        driveList = driveList + drive.Substring(0, 1) + "  Hard disk \r\n";
                        break;
                    case 4:
                        driveList = driveList + drive.Substring(0, 1) + "  Network disk \r\n";
                        break;
                    case 5:
                        driveList = driveList + drive.Substring(0, 1) + "  CD-ROM \r\n";
                        break;
                    case 6:
                        driveList = driveList + drive.Substring(0, 1) + "  RAM driver \r\n";
                        break;
                }
            }
            result = driveList;
        }

        /// <summary>
        /// 指定したアプリケーションを実行します
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdRunApp(string msg, ref string result)
        {
            try
            {
                System.Diagnostics.Process.Start(msg);
                result = "起動しました。";
            }
            catch (Exception ex)
            {
                result = ex.Message;
            }
        }

        /// <summary>
        /// 指定したURLをオープンします
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdGoToURL(string msg, ref string result)
        {
            try
            {
                System.Diagnostics.Process.Start(msg);
                result = "ページを開けました。";
            }
            catch (Exception ex)
            {
                result = ex.Message;
            }
        }

        /// <summary>
        /// Windows バージョンを取得します
        /// このメソッドは渡されたパラメータを使いません
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdWinVer(string msg, ref string result)
        {
            string winVer_Platform = Environment.OSVersion.Platform.ToString();
            string winVer_PlatformV = Environment.OSVersion.Version.Major.ToString();
            string winVer_build = Environment.OSVersion.Version.Build.ToString();
            string winVer = "Platform : " + winVer_Platform + "\r\n" + "PlatformVersion : " + winVer_PlatformV + "\r\n" + "Build : " + winVer_build + "\r\n";
            result = winVer;
        }

        /// <summary>
        /// スクリーンをキャッチして、画像をクライアントに転送します
        /// このメソッドは渡されたパラメータを使いません
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdGetScreen(string msg, ref string result)
        {
            Rectangle rec = Screen.PrimaryScreen.Bounds;

            Bitmap bmp = new Bitmap(rec.Width, rec.Height, PixelFormat.Format32bppArgb);
            using (Graphics g = Graphics.FromImage(bmp))
            {
                g.CopyFromScreen(rec.X, rec.Y, 0, 0, rec.Size, CopyPixelOperation.SourceCopy);
            }

            MemoryStream ms = new MemoryStream();
            bmp.Save(ms, ImageFormat.Jpeg);

            try
            {
                string ip = ((IPEndPoint)_client.Client.RemoteEndPoint).Address.ToString();
                TcpClient c = new TcpClient(ip, 1889);
                NetworkStream netStream = c.GetStream();
                ms.WriteTo(netStream);
                ms.Close();
                netStream.Close();
                c.Close();
                result = "画像送信しました。";
            }
            catch (Exception ex)
            {
                result = ex.Message;
            }
        }

        /// <summary>
        /// メッセージ送信
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdMsg(string msg, ref string result)
        {
            _form.Invoke(new SamDelegate(_form.SetText), msg);
            result = "コマンド実行成功！";
        }

        /// <summary>
        /// CD ドライブ制御する為、Win32APIの呼出
        /// </summary>
        /// <param name="lpstrCommand"></param>
        /// <param name="lpstrReturnstring"></param>
        /// <param name="uReturnLength"></param>
        /// <param name="hwndCallback"></param>
        /// <returns></returns>
        [DllImport("winmm.dll", EntryPoint = "mciSendString", CharSet = CharSet.Auto)]
        public static extern int mciSendString(string lpstrCommand, string lpstrReturnstring, int uReturnLength, int hwndCallback);

        /// <summary>
        /// サーバのCDドライブオープン/クローズ
        /// (CDドライブ存在すれば)
        /// このメソッドは渡されたパラメータを使いません
        /// </summary>
        /// <param name="msg">受信したメッセージ</param>
        /// <param name="result">実行結果</param>
        private void CmdCdControl(string msg, ref string result)
        {
            if (!DoorOpened)
                mciSendString("set cdaudio door open", "", 0, 0);
            if (DoorOpened)
                mciSendString("set cdaudio door closed", "", 0, 0);
            DoorOpened = !DoorOpened;
            result = "コマンド実行しました。";
        }
    }
}
